/*
 *  Vazaar
 *  file: items.c
 *
 *
 *  Copyright (C) 2005 Tomás Vírseda <kaskaras@yahoo.es>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gnome.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <glib.h>
#include <glade/glade.h>
#include <libxml/xmlreader.h>
#include <libgnomevfs/gnome-vfs.h>
#include <libgnomevfs/gnome-vfs-mime-utils.h>
#include <libgnomevfs/gnome-vfs-mime-handlers.h>
#include <gtkmozembed.h>
#include <gtksourceview/gtksourceview.h>
#include <gtksourceview/gtksourcebuffer.h>
#include <gtksourceview/gtksourcelanguage.h>
#include <gtksourceview/gtksourcelanguagesmanager.h>

#include "categories.h"
#include "items.h"
#include "vazaar.h"
#include "debug.h"
#include "support.h"
#include "callbacks.h"

#define DRUID_ITEM_DIALOG					"winItem"

#define DRUID_DELETE_SIGNAL					"on_winItem_delete_event"
#define DRUID_CANCEL_SIGNAL					"on_druidItem_cancel"
#define DRUID_BACK_SIGNAL					"on_druidItem_back"
#define DRUID_NEXT_SIGNAL					"on_druidItem_next"
#define DRUID_FINISH_SIGNAL					"on_druidItem_ok"
#define ITEMNAME_CHANGED_SIGNAL				"on_druidItem_name_changed"
#define TRVCAT_CURSOR_CHANGED_SIGNAL		"on_druidItem_treecat_cursor_changed"
#define TREETYPES_CURSOR_CHANGED_SIGNAL		"on_druidItem_treetypes_cursor_changed"
#define URI_GENERIC_CHANGED					"on_druidItem_uri_pixmap_changed"
#define ENTRY_URI_GENERIC_CHANGED			"on_druidItem_uri_pixmap_changed"
#define URI_PIXMAP_CHANGED					"on_druidItem_uri_generic_changed"
#define LOCAL_SELECTED_FILE					"on_rbLocal_clicked"
#define REMOTE_SELECTED_FILE				"on_rbRemote_clicked"
#define COPYPASTE_SELECTED_FILE				"on_rbCopypaste_clicked"
#define URI_REMOTE_CHANGED					"on_uri_remote_changed"

#define READ_BUFFER_SIZE   4096


extern xmlDocPtr items_doc;

gboolean
vazaar_cmd_item_belongto(gchar * category, GSList * categories)
{
    gboolean        belong = FALSE;
    GSList         *aux;
    gint            res;
    aux = categories;
    while ((aux) && (!belong)) {
		res = vazaar_cmd_strings_compare((gchar *) aux->data, category);
		if (res == 0)
			belong = TRUE;
		aux = aux->next;
    }

    return belong;
}


GtkTreeModel   *
vazaar_gui_itemmodel_create()
{
    GtkTreeSortable *sortable;
    GtkTreeStore   *model = NULL;

    model = gtk_tree_store_new(	ITEM_NUM_COLS,	/* Number of columns */
								GDK_TYPE_PIXBUF, /* Icon for mime type */		
								G_TYPE_STRING,	/* md5 id */
								G_TYPE_STRING,	/* Name */
								G_TYPE_STRING,	/* Description */
								G_TYPE_STRING,	/* Mime */
								G_TYPE_STRING,	/* Date Start */
								G_TYPE_STRING);	/* Date Modified */
	
    sortable = GTK_TREE_SORTABLE(model);
    gtk_tree_sortable_set_sort_column_id(sortable, ITEM_COL_NOM,
					 GTK_SORT_ASCENDING);
    g_assert(model);
    if (DEBUG)
		g_message(_("%s => Item model created"), __FUNCTION__);
    return GTK_TREE_MODEL(model);
}


gint
vazaar_gui_itemmodel_fill(Vazaar *app, GtkTreeModel * model, gchar * category, GList **litems)
{
	GdkPixbuf		*icon = NULL;
	const gchar			*nicon = NULL;
    GtkTreeIter     parent = { 0 };
    xmlNodePtr      root,
                    child;
    VazaarItem     *item;
	VazaarItemSort *item_sort;
    gint            nitems = 0;
	GError			*error = NULL;

    item = (VazaarItem *) NULL;
	item_sort = (VazaarItemSort *) NULL;
    gtk_tree_store_clear(GTK_TREE_STORE(model));
    root = xmlDocGetRootElement(items_doc);
    child = root->xmlChildrenNode;
    while (child != NULL) {
		item_sort = g_new0 (VazaarItemSort, 1);
		item = vazaar_xml_item_get(child);

		if (vazaar_cmd_item_belongto(category, item->categories)) {
			/* Try to load from cache. */
			icon = load_mime_icon (app, item->type);
			//icon = get_pixbuf_from_mime_type (item->type, 32);
			//icon = NULL;
			gtk_tree_store_append(GTK_TREE_STORE(model), &parent, NULL);
			gtk_tree_store_set(	GTK_TREE_STORE(model), 
								&parent, 
								ITEM_COL_IMG, icon,
								ITEM_COL_ID, item->id, 
								ITEM_COL_NOM, item->name,
			//					ITEM_COL_DES, item->desc,
			//		   			ITEM_COL_MIME, gnome_vfs_mime_get_description (item->type), 
								ITEM_COL_DATES, item->dates,
								ITEM_COL_DATEM, item->datem,
								-1);
			nitems++;
			item_sort->item = item;
			item_sort->iter = parent;
			*litems = g_list_append (*litems, item_sort);
		}
		child = child->next;
    }
    if (DEBUG)
		g_message("%s => Filling item model", __FUNCTION__);

    return nitems;
}

void
vazaar_gui_itemmodel_createview(GtkTreeModel * model,
				GtkTreeView * treeview)
{
    GtkTreeViewColumn	*column_id,
						*column_icon,						
                   		*column_name;
	//					*column_desc,
    //             		*column_mime,
	//					*column_dates,
	//					*column_datem;
    GtkCellRenderer		*cell_id,
						*cell_icon,
						*cell_name;                   		
	//					*cell_desc,
    //               	*cell_mime,
	//					*cell_dates,
	//					*cell_datem;

	
    /*
     * ITEM ICON COLUMN 
     */
	
    column_icon = gtk_tree_view_column_new();
    gtk_tree_view_column_set_resizable(column_icon, FALSE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_icon);
    cell_icon = gtk_cell_renderer_pixbuf_new();
    gtk_tree_view_column_pack_end(column_icon, cell_icon, TRUE);
    gtk_tree_view_column_add_attribute(column_icon, cell_icon, "pixbuf", ITEM_COL_IMG);

    /*
     * ITEM ID COLUMN 
     */
    column_id = gtk_tree_view_column_new();
    gtk_tree_view_column_set_resizable(column_id, TRUE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_id);
    cell_id = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_end(column_id, cell_id, TRUE);
    gtk_tree_view_column_add_attribute(column_id, cell_id, "text", ITEM_COL_ID);
    gtk_tree_view_column_add_attribute(column_id, cell_id, "visible", ITEM_COL_ID);
    gtk_tree_view_column_set_visible(column_id, FALSE);

    /*
     * ITEM DATE MODIFIED
     */
	 /*
    column_datem = gtk_tree_view_column_new();
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_datem);
    cell_datem = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_end(column_datem, cell_datem, TRUE);
    gtk_tree_view_column_add_attribute(column_datem, cell_datem, "text", ITEM_COL_DATEM);
	*/
    /*
     * ITEM NAME COLUMN 
     */
    column_name = gtk_tree_view_column_new();
    gtk_tree_view_column_set_resizable(column_name, TRUE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_name);
    cell_name = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_end(column_name, cell_name, TRUE);
    gtk_tree_view_column_add_attribute(column_name, cell_name, "text", ITEM_COL_NOM);
	//gtk_tree_view_column_set_clickable (column_name, TRUE);

    /*
     * ITEM DESC COLUMN 
     */
	 /*
    column_desc = gtk_tree_view_column_new();
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_desc);
    cell_desc = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_end(column_desc, cell_desc, TRUE);
    gtk_tree_view_column_add_attribute(column_desc, cell_desc, "text", ITEM_COL_DES);
	gtk_tree_view_column_set_sizing (column_desc, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
    //gtk_tree_view_column_add_attribute(column_desc, cell_desc, "visible", ITEM_COL_DES);
    //gtk_tree_view_column_set_visible(column_desc, FALSE);
*/
    /*
     * ITEM MIME COLUMN 
     */
	 /*
    column_mime = gtk_tree_view_column_new();
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_mime);
    cell_mime = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_end(column_mime, cell_mime, TRUE);
    gtk_tree_view_column_add_attribute(column_mime, cell_mime, "text", ITEM_COL_MIME);
*/
    /*
     * ITEM DATE START 
     */
	 /*
    column_dates = gtk_tree_view_column_new();
    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column_dates);
    cell_dates = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_end(column_dates, cell_dates, TRUE);
    gtk_tree_view_column_add_attribute(column_dates, cell_dates, "text", ITEM_COL_DATES);
	//gtk_tree_view_column_set_clickable (column_dates, TRUE);
	*/
	
	
    gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), GTK_TREE_MODEL(model));
    gtk_tree_selection_set_mode(gtk_tree_view_get_selection (	GTK_TREE_VIEW(treeview)),
																GTK_SELECTION_BROWSE);
				
				
	gtk_tree_view_column_set_title (column_name, _("Name"));
//	gtk_tree_view_column_set_title (column_desc, _("Description"));
//	gtk_tree_view_column_set_title (column_mime, _("Mime type"));
//	gtk_tree_view_column_set_title (column_dates, _("Created"));
	//gtk_tree_view_column_set_title (column_datem, _("Modified"));
	
	
    if (DEBUG)
	g_message("%s => Item model view created", __FUNCTION__);
}



void
vazaar_gui_itemmodel_refresh(GtkTreeView * treeitem)
{
    GtkTreeModel   *model;

    model = gtk_tree_view_get_model(treeitem);

}



gint
vazaar_gui_items_load(Vazaar * app, gchar * category)
{
    GtkTreeModel   *model;
    gint            nitems = 0;

    model = gtk_tree_view_get_model(GTK_TREE_VIEW (app->treeitem));

    if (model) {
		nitems = vazaar_gui_itemmodel_fill(app, model, category, &app->current_items);
		GList *aux = app->current_items;
    } 
	else {
		g_error(_("Categories model doesn't exists"));
    }

    return nitems;
}

void vazaar_xml_item_change_category (gchar *old_cat, gchar *new_cat) {
	VazaarItem *item;
	xmlNodePtr root, child;
	GSList *founds = NULL;
	
    root = xmlDocGetRootElement(items_doc);
    child = root->xmlChildrenNode;
	
    while (child != NULL) {
		item = vazaar_xml_item_get(child);
		if (vazaar_cmd_item_belongto(old_cat, item->categories)) {
			founds = g_slist_append (founds, child);
		}
		child = child->next;
		g_free (item);
	}
	
	while (founds) {
		item = vazaar_xml_item_get ( (xmlNodePtr)founds->data );
		vazaar_xml_item_delete (item->id);
	
		xmlNodePtr node, rootcat, nodecat;
		
		node = founds->data;
		rootcat = node->xmlChildrenNode;
		nodecat = rootcat->xmlChildrenNode;
		while (nodecat) {
			if (vazaar_cmd_strings_compare ( xmlNodeGetContent(nodecat), old_cat) == 0) {
				xmlUnlinkNode (nodecat);
				xmlNewChild(rootcat, NULL, "category", new_cat);
			}
			nodecat = nodecat->next;
		}
		g_free (item);
		item = vazaar_xml_item_get (node);
		vazaar_xml_item_add (item);
		founds = founds->next;
	}
	vazaar_xml_items_savefile ();		
}


VazaarItem     *
vazaar_xml_item_get(xmlNodePtr node)
{
    VazaarItem     *item;
    xmlNodePtr      rootcat,
                    nodecat;

    item = g_new0(VazaarItem, 1);
    item->id = g_strdup(xmlGetProp(node, "id"));
    item->type = g_strdup(xmlGetProp(node, "type"));
    item->name = g_strdup(xmlGetProp(node, "name"));
    item->desc = g_strdup(xmlGetProp(node, "desc"));
    item->data = g_strdup(xmlGetProp(node, "data"));
	item->dates = g_strdup(xmlGetProp(node, "dates"));
	item->datem = g_strdup(xmlGetProp(node, "datem"));
    item->extra = g_strdup(xmlGetProp(node, "extra"));

    rootcat = node->xmlChildrenNode;
    nodecat = rootcat->xmlChildrenNode;
    while (nodecat) {
		item->categories = g_slist_append(item->categories, xmlNodeGetContent(nodecat));
		nodecat = nodecat->next;
    }
    if (DEBUG) {
		g_message("%s => %s", __FUNCTION__, item->name);
    }
	//g_message ("Date Start: %s\tDate Modified: %s", item->dates, item->datem);	
	
    return item;
}

void
vazaar_xml_item_add(VazaarItem * item)
{
    xmlNodePtr      items_root;
    xmlNodePtr      xitem,
                    xitemCats;
    GSList         *aux = NULL;

    items_root = xmlDocGetRootElement(items_doc);
    xitem = xmlNewChild(items_root, NULL, "item", NULL);

    xmlSetProp(xitem, "id", item->id);
    xmlSetProp(xitem, "type", item->type);
    xmlSetProp(xitem, "name", item->name);
    xmlSetProp(xitem, "desc", item->desc);
    xmlSetProp(xitem, "data", item->data);
	xmlSetProp(xitem, "dates", item->dates);
	xmlSetProp(xitem, "datem", item->datem);
    xmlSetProp(xitem, "extra", item->extra);

    xitemCats = xmlNewChild(xitem, NULL, "categories", NULL);

    aux = item->categories;
    while (aux) {
		xmlNewChild(xitemCats, NULL, "category", (gchar *) aux->data);
		aux = aux->next;
    }

    vazaar_xml_items_savefile();
    if (DEBUG)
	g_message("%s => Cat: %s", __FUNCTION__, item->name);
}

xmlNodePtr
vazaar_xml_item_set_node (VazaarItem * item)
{
    xmlNodePtr      items_root;
    xmlNodePtr      xitem,
                    xitemCats;
    GSList         *aux = NULL;

    items_root = xmlDocGetRootElement(items_doc);
    xitem = xmlNewChild(items_root, NULL, "item", NULL);

    xmlSetProp(xitem, "id", item->id);
    xmlSetProp(xitem, "type", item->type);
    xmlSetProp(xitem, "name", item->name);
    xmlSetProp(xitem, "desc", item->desc);
    xmlSetProp(xitem, "data", item->data);
    xmlSetProp(xitem, "extra", item->extra);

    xitemCats = xmlNewChild(xitem, NULL, "categories", NULL);

    aux = item->categories;
    while (aux) {
		xmlNewChild(xitemCats, NULL, "category", (gchar *) aux->data);
		aux = aux->next;
    }

    vazaar_xml_items_savefile();
    if (DEBUG)
		g_message("%s => Cat: %s", __FUNCTION__, item->name);
	
	return xitem;
}

gboolean
vazaar_xml_item_delete(const gchar * id)
{
    gboolean        found = FALSE;
    xmlNodePtr      root,
                    child;
    VazaarItem     *item = NULL;

    root = xmlDocGetRootElement(items_doc);
    child = root->xmlChildrenNode;

    while ((child != NULL) && (!found)) {
	item = vazaar_xml_item_get(child);
	if (vazaar_cmd_strings_compare(item->id, g_strdup(id)) == 0) {
	    found = TRUE;
	}
	if (!found)
	    child = child->next;
    }
    if (found) {
		xmlUnlinkNode(child);
		vazaar_xml_items_savefile();
    }
    return found;
}

void
vazaar_cmd_item_delete(const gchar * item_id)
{
    vazaar_xml_item_delete(item_id);
    gnome_vfs_unlink(vazaar_cmd_item_file_id_get(item_id));
}

void
vazaar_xml_item_edit(xmlNodePtr old, xmlNodePtr new)
{

}


xmlNodePtr
vazaar_xml_item_find(gchar * id)
{
    xmlNodePtr      child,
                    root,
                    node;
    gboolean        found;
    VazaarItem     *item;
    found = FALSE;

    node = NULL;
    root = xmlDocGetRootElement(items_doc);
    child = root->xmlChildrenNode;
    while ((child) && (!found)) {
		item = vazaar_xml_item_get(child);
		if (g_utf8_collate (item->id, id) == 0) {
			found = TRUE;
			node = child;
		}
		child = child->next;
    }

    return node;
}

int
vazaar_xml_items_savefile()
{
    gchar          *doc_path;
    int             res;

    doc_path = vazaar_xml_itemdoc_get();
    res = xmlSaveFile(doc_path, items_doc);
    g_free(doc_path);
	
    return res;
}



void
vazaar_gui_druiditem_new(Vazaar * app, VazaarItem * item, gchar *data_dragged)
{
    GtkTreeModel   *modelcat;
    GtkTreeViewColumn *col_cat_chk;
    VazaarDruidItem *this;
	const gchar *str = NULL;
	gchar				*uri_str;
	GnomeVFSURI			*uri_vfs;
	
    this = g_new0(VazaarDruidItem, 1);
	
    this->app = app;
    this->xml =
	glade_xml_new(GLADE_VAZAAR_PROJECT_FILE, DRUID_ITEM_DIALOG, NULL);
    this->dialog = glade_xml_get_widget(this->xml, DRUID_ITEM_DIALOG);
    this->wparent = app->window;
	this->treeviewcat = glade_xml_get_widget(this->xml, "trvcat");
    this->itemName = glade_xml_get_widget(this->xml, "itemName");
    this->itemDesc = glade_xml_get_widget(this->xml, "itemDesc");
	this->rbLocal = glade_xml_get_widget (this->xml, "rbLocal");
	this->rbRemote = glade_xml_get_widget (this->xml, "rbRemote");
	this->rbCopypaste = glade_xml_get_widget (this->xml, "rbCopypaste");
	this->frmLocal = glade_xml_get_widget (this->xml, "frmLocal");
	this->frmRemote = glade_xml_get_widget (this->xml, "frmRemote");
	this->frmCopypaste = glade_xml_get_widget (this->xml, "frmCopypaste");
	this->check_remote = glade_xml_get_widget (this->xml, "href_remote");
	this->uri_remote = glade_xml_get_widget (this->xml, "uri_remote");
	this->uri_generic = glade_xml_get_widget(this->xml, "uri_generic");
	this->entry_uri_generic = glade_xml_get_widget(this->xml, "entry_uri_generic");
	this->copypaste =  glade_xml_get_widget(this->xml, "txvCopypaste");
	this->btnOK =   glade_xml_get_widget(this->xml, "btnOK");
	this->btnCancel =   glade_xml_get_widget(this->xml, "btnCancel");
    this->new_item = g_new0(VazaarItem, 1);
    this->edit = FALSE;
    vazaar_gui_druiditem_connect_signals(this);
	
    gtk_window_set_transient_for(GTK_WINDOW(this->dialog), GTK_WINDOW(app->window));
	gtk_window_set_focus            (GTK_WINDOW(this->dialog), this->dialog);
	gtk_window_set_keep_above		(GTK_WINDOW(this->dialog), TRUE);
    /*
     * Fill cat model and show check column
     */
    modelcat = vazaar_gui_catmodel_create_and_fill();
	g_assert (modelcat);
	vazaar_gui_catmodel_createview(modelcat, GTK_TREE_VIEW(this->treeviewcat));
	col_cat_chk = gtk_tree_view_get_column(GTK_TREE_VIEW(this->treeviewcat), 1);
    gtk_tree_view_column_set_visible(col_cat_chk, TRUE);
	
    this->cat_selected = NULL;

	/* Set monospace font to CopyPaste widget */
	set_monospace_font (this->copypaste);

	/* If there is a drag'n'drop action ... */
	/* Reorganizar este codigo por que solo lo entiendo yo */
	if (data_dragged) {
		uri_vfs = gnome_vfs_uri_new (data_dragged);
		
		if (gnome_vfs_uri_is_local (uri_vfs)) {
			uri_str = gnome_vfs_uri_to_string (uri_vfs, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
			gtk_entry_set_text (GTK_ENTRY (this->entry_uri_generic), uri_str);
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (this->rbLocal), TRUE);
			gtk_widget_hide (GTK_WIDGET (this->frmCopypaste));
			gtk_widget_show (GTK_WIDGET (this->frmRemote));			
		}
		else {
			gtk_entry_set_text (GTK_ENTRY (this->uri_remote), data_dragged);
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (this->rbRemote), TRUE);
			gtk_widget_hide (GTK_WIDGET (this->frmCopypaste));
			gtk_widget_show (GTK_WIDGET (this->frmLocal));						
		}
	}

    if (item) {
		this->new_item = item;
		this->edit = TRUE;
    }

    if (this->edit) {
		GtkTextBuffer  *buffer_desc = gtk_text_buffer_new(NULL);
		gtk_entry_set_text (GTK_ENTRY (this->itemName), item->name);
		gtk_text_buffer_set_text(buffer_desc, item->desc, -1);
		gtk_text_view_set_buffer(GTK_TEXT_VIEW (this->itemDesc), buffer_desc);
		
		if (g_utf8_strlen (item->data, -1) > 0) {
			gtk_entry_set_text (GTK_ENTRY (this->uri_remote), item->data);
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (this->rbRemote), TRUE);
		}
		else {
			if	(	(g_utf8_strlen (item->extra, -1) > 0) ||
					(	( (g_utf8_strlen (item->extra, -1) == 0) && 
					 	  (g_utf8_strlen (item->data, -1)  == 0))
					)
				) {
				gchar *supertype = vazaar_cmd_item_get_mime_supertype (item->type);
				if	(	(!vazaar_cmd_item_filter_bad_mimes (item->type)) ||
						(g_utf8_collate (supertype, "image") != 0) ) {
					gchar *content;
					GtkTextBuffer  *buffer = gtk_text_buffer_new(NULL);
					content = vazaar_cmd_item_content_get (item);
					gtk_text_buffer_set_text(buffer, content, -1);
					gtk_text_view_set_buffer(GTK_TEXT_VIEW(this->copypaste), buffer);
					gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(this->rbCopypaste), TRUE);									
				}
				else {			
					gnome_file_entry_set_filename (GNOME_FILE_ENTRY (this->uri_generic), item->extra);
					gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (this->rbLocal), TRUE);
				}
			}
		}
		GSList *aux = item->categories;
		while (aux) {
			gtk_tree_model_foreach	(	modelcat, 
									(	GtkTreeModelForeachFunc)vazaar_gui_foreach_cat_in_item,
			   							aux->data);
			aux = aux->next;
		}
		gchar *str = g_strdup_printf (_("Editing '%s' item"), item->name);
		gnome_appbar_set_status (GNOME_APPBAR (this->app->statusbar), str);		
		g_free (str);
    } 
	else {
		gnome_appbar_set_status (GNOME_APPBAR (this->app->statusbar), _("Adding new item"));
    }
	
	vazaar_cmd_druiditem_check (NULL, this);
	gtk_window_set_transient_for(GTK_WINDOW(this->dialog), GTK_WINDOW(app->window));
    vazaar_gui_druiditem_show(this, this->wparent);
}

void
vazaar_gui_druiditem_show(VazaarDruidItem * this,
			  GtkWidget * parent_window)
{
    g_return_if_fail(this);

    if (this->dialog) {
		
		gtk_widget_set_sensitive(parent_window, FALSE);
		gtk_widget_show (this->dialog);
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(this->rbLocal))) {
			gtk_widget_show (this->frmLocal);
			gtk_widget_hide (this->frmRemote);
			gtk_widget_hide (this->frmCopypaste);
		}
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(this->rbRemote))) {
			gtk_widget_show (this->frmRemote);
			gtk_widget_hide (this->frmLocal);
			gtk_widget_hide (this->frmCopypaste);
		}
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(this->rbCopypaste))) {
			gtk_widget_show (this->frmCopypaste);
			gtk_widget_hide (this->frmRemote);
			gtk_widget_hide (this->frmLocal);
		}
    }
		
}

void
vazaar_gui_item_add(Vazaar * app)
{
    vazaar_gui_druiditem_new(app, NULL, NULL);
}



void
vazaar_gui_item_edit(Vazaar * app)
{
    xmlNodePtr      found;
    GtkTreeSelection *selection;
    GtkTreeModel   *model;
    GtkTreeIter     iter;
    gchar          *id;
    VazaarItem     *item;

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (app->treeitem));
    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, ITEM_COL_ID, &id, -1);
		found = vazaar_xml_item_find(id);
		if (found) {
			item = vazaar_xml_item_get(found);
			vazaar_gui_druiditem_new(app, item, NULL);
		}
    }
}

void
vazaar_gui_item_del(Vazaar * app)
{
    GtkTreeSelection *selection;
    GtkTreeModel   *model;
    GtkTreeIter     iter;
    GtkWidget      *dialog;
    gchar          *title,
					*details,
                   *name,
                   *item_id;
    gint            result;

    /*
     * This will only work in single or browse selection mode! 
     */
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (app->treeitem));
    if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
	gtk_tree_model_get	(	model, &iter, 
							ITEM_COL_ID, &item_id,
			   				ITEM_COL_NOM, &name, 
							-1);

	if (gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON(app->wastebasket))) {
		title = g_strdup_printf ( _("Are you sure?"));
		details = g_strdup_printf ( _("This item will be deleted forever"));		
	}
	else {
		title = g_strdup_printf(	_("Delete '%s' item?"), name);
		details = g_strdup_printf ( _("This item will be deleted from all categories and stored in the wastebasket.\n\n" \
							"If you only want quit from this category, edit this item"));
	}
	result = vazaar_gui_show_dialog (app->window, GTK_STOCK_DIALOG_QUESTION, title, details);
	switch (result) {
	case GTK_RESPONSE_OK:
	    model = gtk_tree_view_get_model(GTK_TREE_VIEW (app->treeitem));
	    gtk_tree_store_remove(GTK_TREE_STORE(model), &iter);
		if (gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON(app->wastebasket))) {
	    	vazaar_cmd_item_delete(item_id);
			vazaar_gui_wastebasket_toggled (NULL, app);
		}
		else {
			vazaar_cmd_move_item_to_wastebasket (app, item_id);
			on_trvVazaarCat_cursor_changed (NULL, app);
		}
		gnome_appbar_set_status (GNOME_APPBAR (app->statusbar), _("Item deleted."));
	    break;
	case GTK_RESPONSE_CANCEL:
		gnome_appbar_set_status (GNOME_APPBAR (app->statusbar), _("Item not deleted."));
	    break;
	}
	g_free(name);
	g_free(item_id);
	g_free(title);
	g_free(details);
    } 
	else {
		if (DEBUG)
	   		g_message("no row selected.\n");
    }
}

void
vazaar_gui_druiditem_destroy(VazaarDruidItem * this)
{
	GtkWidget	*widget;
	
    g_return_if_fail(this != NULL);

    gtk_widget_destroy(this->dialog);
    gtk_widget_set_sensitive(this->wparent, TRUE);
    g_slist_free(this->cat_selected);
    g_free(this->new_item);
    g_free(this);
	if (gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON(this->app->wastebasket))) {
		vazaar_gui_wastebasket_toggled (NULL, this->app);
	}
	else {
		/*
		*/
		on_trvVazaarCat_cursor_changed(NULL, this->app);
		
	}
}

//ERROR AL VER LA PAPELERA SIN ITEMS

void
vazaar_gui_druiditem_delete_event	(	GtkWindow * win,
				  						VazaarDruidItem * druiditem)
{
	return;
}

void
vazaar_gui_druiditem_connect_signals(VazaarDruidItem * this)
{
    g_return_if_fail(this != NULL);

    glade_xml_signal_connect_data	(	this->xml, 
										DRUID_DELETE_SIGNAL,
				  						GTK_SIGNAL_FUNC(vazaar_gui_druiditem_delete_event),
				  						this);
	
    glade_xml_signal_connect_data	(	this->xml, 
										DRUID_CANCEL_SIGNAL,
				  						GTK_SIGNAL_FUNC (vazaar_gui_druiditem_cancel), 
										this);
	
    glade_xml_signal_connect_data	(	this->xml, 
										ITEMNAME_CHANGED_SIGNAL,
										GTK_SIGNAL_FUNC (vazaar_cmd_druiditem_check),
				  						this);
	
    glade_xml_signal_connect_data	(	this->xml, 
										TRVCAT_CURSOR_CHANGED_SIGNAL,
				  						GTK_SIGNAL_FUNC (vazaar_cmd_druiditem_check),
				  						this);
										
    glade_xml_signal_connect_data	(	this->xml, 
										URI_GENERIC_CHANGED,
				  						GTK_SIGNAL_FUNC (vazaar_cmd_druiditem_check),
				  						this);

    glade_xml_signal_connect_data	(	this->xml, 
										ENTRY_URI_GENERIC_CHANGED,
				  						GTK_SIGNAL_FUNC (vazaar_cmd_druiditem_check),
				  						this);
										
    glade_xml_signal_connect_data	(	this->xml, 
										DRUID_FINISH_SIGNAL, 
										GTK_SIGNAL_FUNC (vazaar_gui_druiditem_finish), 
										this);
										
    glade_xml_signal_connect_data	(	this->xml, 
										URI_PIXMAP_CHANGED,
				  						GTK_SIGNAL_FUNC (vazaar_gui_druiditem_uri_changed),
				  						this);
										
    glade_xml_signal_connect_data	(	this->xml, 
										LOCAL_SELECTED_FILE,
				  						GTK_SIGNAL_FUNC (vazaar_gui_item_select_mode),
				  						this);
										
    glade_xml_signal_connect_data	(	this->xml, 
										REMOTE_SELECTED_FILE,
				  						GTK_SIGNAL_FUNC (vazaar_gui_item_select_mode),
				  						this);
										
    glade_xml_signal_connect_data	(	this->xml, 
										COPYPASTE_SELECTED_FILE,
				  						GTK_SIGNAL_FUNC (vazaar_gui_item_select_mode),
				  						this);
										
    glade_xml_signal_connect_data	(	this->xml, 
										URI_REMOTE_CHANGED,
				  						GTK_SIGNAL_FUNC (vazaar_gui_item_change_href),
				  						this);
}

void vazaar_gui_item_change_href		(	GtkWidget *nothing,
											VazaarDruidItem *this) {
	const gchar *uri;
												
	uri = gtk_entry_get_text (GTK_ENTRY (this->uri_remote));
	gnome_href_set_url (GNOME_HREF (this->check_remote), uri);
}

void 
vazaar_gui_item_select_mode				(	GtkWidget *nothing,
											VazaarDruidItem *this) {

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(this->rbLocal))) {
		gtk_widget_show (this->frmLocal);
		gtk_widget_hide (this->frmRemote);
		gtk_widget_hide (this->frmCopypaste);
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(this->rbRemote))) {
		gtk_widget_show (this->frmRemote);
		gtk_widget_hide (this->frmLocal);
		gtk_widget_hide (this->frmCopypaste);
	}
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(this->rbCopypaste))) {
		gtk_widget_show (this->frmCopypaste);
		gtk_widget_hide (this->frmRemote);
		gtk_widget_hide (this->frmLocal);
	}
	
	vazaar_cmd_druiditem_check (NULL, this);
}

void
vazaar_gui_druiditem_cancel(GtkWidget * window, VazaarDruidItem * this)
{
    g_return_if_fail(this != NULL);
	gnome_appbar_set_status (GNOME_APPBAR(this->app->statusbar), _("Item not added"));
    vazaar_gui_druiditem_destroy(this);
}

gboolean
vazaar_gui_druiditem_name_changed(GtkWidget * entry,
				  VazaarDruidItem * this)
{
	gint		 	 len;
	const gchar		*name = gtk_entry_get_text(GTK_ENTRY (this->itemName));
	gchar			*str;
	
	len = g_utf8_strlen (name, -1);
	if (len > 0) {
		str = g_strdup_printf ("*%s", name);
		gtk_window_set_title (GTK_WINDOW(this->dialog), str);
		return TRUE;
	}
	else {
		if (this->edit) {
			str = g_strdup ("*Unnamed item");
		}
		else {
			str = g_strdup ("Unsaved item");
		}
		gtk_window_set_title (GTK_WINDOW(this->dialog), str);
		return FALSE;
	}
}


gboolean
vazaar_gui_druiditem_treecat_cursor_changed(GtkWidget * window,
					    VazaarDruidItem * this)
{
	GtkTreeModel   *model;
	
	this->cat_selected = NULL;
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(this->treeviewcat));
	g_assert(model);

	gtk_tree_model_foreach(model, (GtkTreeModelForeachFunc)
				   vazaar_gui_foreach_selected_cats,
				   &this->cat_selected);
	if (g_slist_length(this->cat_selected) > 0) {
		return TRUE;
	}
	else {
		return FALSE;
	}
	return FALSE;
}



gboolean
vazaar_gui_druiditem_uri_changed(GtkWidget * window,
					VazaarDruidItem * this)
{
	gchar	*uri_generic = NULL;
	const gchar	*uri_remote  = NULL;
	
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (this->rbCopypaste))) {
		return TRUE;
	}
	else {
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (this->rbLocal))) {
			uri_generic = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(this->uri_generic), FALSE);
			if (uri_generic) {
				if (g_utf8_strlen (uri_generic, -1) > 0) {
					//uri = gnome_vfs_get_uri_from_local_path (uri_generic);
					//g_free (uri);
					g_free (uri_generic);
					return TRUE;
				}
				else { 
					return FALSE;
				}
			}
		}
		else {
			uri_remote = gtk_entry_get_text (GTK_ENTRY (this->uri_remote));
			if (uri_remote) {
				if (g_utf8_strlen (uri_remote, -1) > 0) {
					return TRUE;
				}
				else
					return FALSE;
			}
		}	
	}
	
	return FALSE;
}


void
vazaar_gui_item_set_info (Vazaar *app, VazaarItem *item) {
	GnomeVFSMimeApplication	*defapp;
	GdkPixbuf				*icon;
	
	if (item) {
		defapp = gnome_vfs_mime_get_default_application (item->type);
		
		/* Set data */
		icon = load_mime_icon (app, item->type);
		gtk_image_set_from_pixbuf (GTK_IMAGE (app->itemInfoIcon), icon);
		
		gtk_label_set_text (GTK_LABEL (app->itemInfoId), item->id);
		gtk_label_set_text (GTK_LABEL (app->itemInfoName), item->name);
		gtk_label_set_text (GTK_LABEL (app->itemInfoDesc), item->desc);
		gtk_label_set_text (GTK_LABEL (app->itemInfoMIME), item->type);
		if (defapp) {
			gtk_label_set_text (GTK_LABEL (app->itemInfoApp), defapp->name);
		}
		else {
			gtk_label_set_text (GTK_LABEL (app->itemInfoApp), "None");
		}
		gtk_label_set_text (GTK_LABEL (app->itemInfoSDate), item->dates);
		gtk_label_set_text (GTK_LABEL (app->itemInfoMDate), item->datem);
	}
}

void
vazaar_cmd_druiditem_check (GtkWidget *window, VazaarDruidItem *this) {
	gboolean	name,
				categories,
				uri;
	
	name		= vazaar_gui_druiditem_name_changed (NULL, this);
	categories	= vazaar_gui_druiditem_treecat_cursor_changed (NULL, this);
	uri			= vazaar_gui_druiditem_uri_changed (NULL, this);
	
	if (DEBUG)
		g_message ("name: %d, categories: %d, uri: %d", name, categories, uri);
	
	if ( (name) && (categories) && (uri) )
		gtk_widget_set_sensitive (this->btnOK, TRUE);
	else
		gtk_widget_set_sensitive (this->btnOK, FALSE);
}

gchar *
vazaar_cmd_item_content_get(VazaarItem * item)
{
    GError        **error = NULL;
    gchar          *content;
    gsize           content_len;

    g_file_get_contents	(	vazaar_cmd_item_file_id_get(item->id), 
							&content,
							&content_len, 
							error);
    return content;
}

VFSItemData *
vazaar_cmd_druiditem_item_store_create(VazaarDruidItem * this)
{
    VFSItemData    *vfs_item;

    vfs_item = g_new0(VFSItemData, 1);

    // URI Input /
    vfs_item->uri_input = gnome_vfs_uri_new(this->new_item->data);
    if (DEBUG) {
		g_message(	"%s: URI_INPUT: %s",	
					__FUNCTION__,
					gnome_vfs_uri_to_string	(	vfs_item->uri_input, 
												GNOME_VFS_URI_HIDE_NONE));
    }
    // URI Output /
    vfs_item->file = vazaar_cmd_item_file_id_get(this->new_item->id);
    vfs_item->uri_output = gnome_vfs_uri_new(vfs_item->file);
    if (DEBUG) {
		g_message(	"%s: URI_OUTPT: %s", 
					__FUNCTION__,
		  			gnome_vfs_uri_to_string(	vfs_item->uri_output,
												GNOME_VFS_URI_HIDE_NONE));
    }
    // Open INPUT URI /
    vfs_item->res = gnome_vfs_open_uri (	&vfs_item->handle_input, 
											vfs_item->uri_input,
			   								GNOME_VFS_OPEN_READ);

    // Create OUTPUT URI @ $HOME./vazaar/store/#md5sum# / 
    if (this->edit) {
		vfs_item->res = gnome_vfs_unlink_from_uri (	vfs_item->uri_output);
		vfs_item->res = gnome_vfs_create_uri (	&vfs_item->handle_output,
				 								vfs_item->uri_output,
				 								GNOME_VFS_OPEN_WRITE, 
												TRUE, 
												0644);
    } 
	else {
		vfs_item->res = gnome_vfs_create_uri (	&vfs_item->handle_output,
				 								vfs_item->uri_output,
				 								GNOME_VFS_OPEN_WRITE, 
												TRUE, 
												0644);
    }

    if (vfs_item->res != GNOME_VFS_OK) {
		gchar			*title,
						*message;

		title = g_strdup_printf	("Error %d", vfs_item->res);
		message = g_strdup_printf ("%s", gnome_vfs_result_to_string(vfs_item->res));
		vazaar_gui_show_dialog	(this->wparent, "%s", title, message);
		g_free(title); g_free (message);
		vazaar_gui_druiditem_destroy(this);
		gnome_vfs_close(vfs_item->handle_output);
		return NULL;
    }

    return vfs_item;
}

void
vazaar_cmd_druiditem_item_store_save(VFSItemData * vfs_item)
{
    GnomeVFSFileSize	 bytes_read,
                    	 bytes_written;
    gchar          		*buffer_input[BYTES_TO_PROCESS];

    vfs_item->res = gnome_vfs_read	(	vfs_item->handle_input, 
										buffer_input,
		       							BYTES_TO_PROCESS, 
										&bytes_read);
	
    while (vfs_item->res != GNOME_VFS_ERROR_EOF) {
		gnome_vfs_write	(	vfs_item->handle_output, 
							buffer_input, 
							bytes_read,
							&bytes_written);
		
		vfs_item->res = gnome_vfs_read	(	vfs_item->handle_input, 
											buffer_input,
			   								BYTES_TO_PROCESS, 
											&bytes_read);
    }
	
    gnome_vfs_close(vfs_item->handle_input);
    gnome_vfs_close(vfs_item->handle_output);
}

/*
VFSItemData *
vazaar_cmd_druiditem_item_store_create(VazaarDruidItem * this)
{
    VFSItemData    *vfs_item;

    vfs_item = g_new0(VFSItemData, 1);

    // URI Input /
    vfs_item->uri_input = gnome_vfs_uri_new(this->new_item->data);

    // URI Output /
    vfs_item->file = vazaar_cmd_item_file_id_get(this->new_item->id);
    vfs_item->uri_output = gnome_vfs_uri_new(vfs_item->file);

    // Open INPUT URI /
	gnome_vfs_async_open_uri	(	&vfs_item->handle_input, 
									vfs_item->uri_input,
									GNOME_VFS_OPEN_READ,
									GNOME_VFS_PRIORITY_DEFAULT,
									NULL,
									NULL);

    // Create OUTPUT URI @ $HOME./vazaar/store/#md5sum# / 
    if (this->edit) {
		vfs_item->res = gnome_vfs_unlink_from_uri (	vfs_item->uri_output);
		gnome_vfs_async_create_uri	(	&vfs_item->handle_output,
				 						vfs_item->uri_output,
				 						GNOME_VFS_OPEN_WRITE, 
										FALSE,
										0644,
										GNOME_VFS_PRIORITY_DEFAULT,
										NULL,
										NULL);
    } 
	else {
		gnome_vfs_async_create_uri	(	&vfs_item->handle_output,
				 						vfs_item->uri_output,
				 						GNOME_VFS_OPEN_WRITE, 
										FALSE,
										0644,
										GNOME_VFS_PRIORITY_DEFAULT,
										NULL,
										NULL);
    }

    if (vfs_item->res != GNOME_VFS_OK) {
		gchar		*title,
					*message;

		vazaar_gui_druiditem_destroy(this);
		return NULL;
    }
	gnome_vfs_async_close	(	vfs_item->handle_output,
								NULL,
								NULL);
	
	//gnome_vfs_close(vfs_item->handle_output);
    return vfs_item;
}

void
vazaar_cmd_druiditem_item_store_save(VFSItemData * vfs_item)
{
    GnomeVFSFileSize	 bytes_read,
                    	 bytes_written;
    gchar          		*buffer_input[BYTES_TO_PROCESS];
	
	g_message ("gnome_vfs_async_create_uri");
	exit (-1);
	gnome_vfs_async_read	(	vfs_item->handle_input, 
								buffer_input,
								BYTES_TO_PROCESS,
								(GnomeVFSAsyncReadCallback)read_cb,
								NULL);

	//while (vfs_item->res != GNOME_VFS_ERROR_EOF) {
    while (1) {
		gnome_vfs_async_write	(	vfs_item->handle_output,
									buffer_input,
									BYTES_TO_PROCESS,
									(GnomeVFSAsyncWriteCallback) write_cb,
									NULL);
		gnome_vfs_async_read	(	vfs_item->handle_input, 
									buffer_input,
									BYTES_TO_PROCESS,
									NULL,
									NULL);
    }
	
    gnome_vfs_async_close(vfs_item->handle_input, NULL, NULL);
    gnome_vfs_async_close(vfs_item->handle_output, NULL, NULL);
}
*/

void
vazaar_gui_druiditem_finish(GtkWidget * window, VazaarDruidItem * this)
{
	GnomeVFSURI *uri;
    VFSItemData    *vfs_item = NULL;
	xmlNodePtr		node;
	
	g_assert (this);

	if (vazaar_gui_druiditem_get_data (this)) {
		node = vazaar_xml_item_find (this->new_item->id);
		if ((node) && (!this->edit)) {
			vazaar_gui_show_dialog (this->wparent, GTK_STOCK_DIALOG_ERROR, _("There is another item with the same name"), _("You must choose another"));
			return;
		}
		
		if (this->edit) {
			vazaar_cmd_item_delete (this->new_item->id);
		}
		
		/*
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(this->rbCopypaste))) {
			
		}
	*/
		uri = gnome_vfs_uri_new (this->new_item->data);
		if (gnome_vfs_uri_is_local (uri)) {
			if (vazaar_cmd_strings_compare (this->new_item->type, "text/html") != 0) {
				vfs_item = vazaar_cmd_druiditem_item_store_create (this);
				vazaar_cmd_druiditem_item_store_save (vfs_item);
				this->new_item->data = NULL;
			}
		}
	
		vazaar_xml_item_add (this->new_item);
			
		if (DEBUG) {
			g_message(_("Added new item:"));
			g_message ("\tId: %s", this->new_item->id);
			g_message ("\tName: %s", this->new_item->name);
			g_message ("\tDesc: %s", this->new_item->desc);
			g_message ("\tData: %s", this->new_item->data);
			g_message ("\tExtra: %s", this->new_item->extra);
		}	
	}
	vazaar_gui_druiditem_destroy(this);
}


gboolean
vazaar_gui_druiditem_get_data (VazaarDruidItem * this)
{
    GTimeVal        now;
    GString        *stmp = NULL;
    GtkTextBuffer   *buffer;
	GtkTextIter      start, end;
	GnomeVFSFileInfo *info;
	gchar *uri_str;
	
	g_assert (this);
	info = gnome_vfs_file_info_new ();
	/* Abort if it isn't a regular file... */
	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (this->rbCopypaste))) {
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (this->rbLocal))) {
			uri_str = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(this->uri_generic), FALSE);
			gnome_vfs_get_file_info (uri_str, info, GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE);
			if (info->type != GNOME_VFS_FILE_TYPE_REGULAR) {
				gchar *title = g_strdup_printf (_("It was imposible add this item."));
				gchar *details = g_strdup_printf (_("You can only add regular files"));
				vazaar_gui_show_dialog (this->app->window, GTK_STOCK_DIALOG_ERROR, title, details);
				return FALSE;
			}
		}
	}
	
	/* Name and Description */
	this->new_item->name = g_strdup (gtk_entry_get_text (GTK_ENTRY (this->itemName)));
	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (this->itemDesc));
	gtk_text_buffer_get_start_iter (buffer, &start);
	gtk_text_buffer_get_end_iter (buffer, &end);
	this->new_item->desc = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
	
    if (!this->edit) {
		/* Set Date Start */
		this->new_item->dates = vazaar_cmd_get_current_date ();
		this->new_item->datem = vazaar_cmd_get_current_date ();
		/** MD5 SUM ITEM **/
		g_get_current_time(&now);
		stmp = g_string_new (this->new_item->name);
		this->new_item->id = vazaar_cmd_item_md5sum_get(stmp);
		g_string_free(stmp, TRUE);
    } 
	else {
		/* Set Date modified */
		this->new_item->datem = vazaar_cmd_get_current_date ();
    }

	/* File type and content/uri */
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (this->rbCopypaste))) {
		//GtkTextBuffer		*buffer;
		//GtkTextIter			 start, 
		//				 	 end;
		gchar				*content;
		gchar				*filename;
		FILE				*out = NULL;
		
		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(this->copypaste));
		gtk_text_buffer_get_start_iter (buffer, &start);
		gtk_text_buffer_get_end_iter (buffer, &end);
		content = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
		filename = g_strdup_printf ("%s/.%s", g_get_tmp_dir (), this->new_item->id);
		out = fopen (filename, "w");
		fprintf (out, "%s", content);  
		fclose (out);
		this->new_item->data = g_strdup_printf ("%s", filename);
		if (DEBUG) {
			g_message ("Mime 1: %s", gnome_vfs_get_mime_type_from_file_data (gnome_vfs_uri_new (filename)));
			g_message ("Mime 2: %s", gnome_vfs_get_mime_type (filename));
			g_message ("Mime 3: %s", gnome_vfs_get_file_mime_type (filename, NULL, FALSE));
		}		
		if (!this->edit) {
			this->new_item->type = g_strdup (gnome_vfs_get_mime_type (filename));		
		}
	}
	else {
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (this->rbLocal))) {
			this->new_item->data = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(this->uri_generic), FALSE);
			this->new_item->extra = g_strdup (this->new_item->data);
			this->new_item->type = gnome_vfs_get_mime_type (this->new_item->data);
		} 
		else {
			this->new_item->data = g_strdup (gtk_entry_get_text (GTK_ENTRY (this->uri_remote)));
			this->new_item->type = g_strdup_printf ("%s", "text/html");
		}
	}
	if (DEBUG)
		g_message ("Uri: %s (%s)", this->new_item->data, this->new_item->type);
	

	/* Selected categories */
	this->new_item->categories = this->cat_selected;
	
	return TRUE;
}

/*
 * Devuelve la uri donde se almacenara la info del item en el store 
 */
gchar          *
vazaar_cmd_item_file_id_get(const gchar * id)
{
    return g_strdup_printf("%s/%s", vazaar_cmd_storedir_get(), id);
}

gboolean
vazaar_cmd_item_filter_bad_mimes (gchar *mime) {
	gint		 i;
	gboolean	 pass;
	gchar		*supertype;
	
	/* Sometimes, there are mimetypes such as application/x-python that belong to
	   text mimetypes. we need filter application mimetypes */
	gchar		*bad_mime[] = {
		"application/vnd.sun.xml.writer",
		"application/vnd.sun.xml.calc",
		"application/vnd.sun.xml.impress",
		"application/vnd.sun.xml.draw",
		"application/vnd.sun.xml.writer.global",
		"application/x-gnumeric",
		"application/pdf",
		"application/x-executable",
		"application/ogg",
		"application/x-object",
		"application/octet-stream",
		"application/x-sharedlib",		
		"application/x-not-regular-file",
		"application/x-archive",
		"application/x-bzip2",
		"application/x-iso9660",
		"application/x-zip",
		"application/zip",
		"application/x-rar",
		"application/x-dosexec",
		"application/msword",
		"application/postscript",
		"application/x-tar",
		"application/x-debian-package",
		"application/x-coredump",
		"application/x-empty",
		NULL
	};
	
	/* Theses supertypes aren't supported, at the moment, by GNOME Vazaar */
	/* You can check /etc/mime.types */
	gchar		*bad_supertypes[] = {
		"audio",
		"chemical",
		"inode",
		"message",
		"model",
		"multipart",
		"video",
		"x-conference",
		"x-world",
		NULL
	};
		
	supertype = vazaar_cmd_item_get_mime_supertype (mime);	
	i = 0;
	pass = TRUE;
	
	while ( (pass) && (bad_supertypes[i]) ) {
		if (g_utf8_collate (bad_supertypes[i], supertype) == 0) {
			pass = FALSE;
		}
		i++;
	}
	
	i = 0;
	while ( (pass) && (bad_mime[i]) ) {
		if (g_utf8_collate (bad_mime[i], mime) == 0) {
			pass = FALSE;
		}
		i++;
	}
	
	return pass;
}

void vazaar_gui_item_get_real (Vazaar *app, VazaarItem *item) {
	GtkWidget		*widget;
	gchar			*supertype; // supertype MIME
	gboolean		 pass = FALSE;
	gchar			*title; // Window title	
	
	/* Set current item */
	app->citem = g_strdup (item->id);

	/* Set window title */
	title = g_strdup_printf ("%s", item->name);				
	gtk_window_set_title (GTK_WINDOW(app->window), title);	
	g_free (title);
	
	/* Set item info panel */
	vazaar_gui_item_set_info (app, item);
	
	//* Delete any widget plugged
	widget = gtk_paned_get_child2(GTK_PANED(app->vpitem));
	if (widget) {
		g_object_ref(app->vpitem);
		gtk_container_remove(GTK_CONTAINER(app->vpitem), widget);
	}	
	
	if (vazaar_cmd_item_filter_bad_mimes (item->type)) {
		/* Get supertype */
		supertype = vazaar_cmd_item_get_mime_supertype (item->type);	

		/* Text */
		if	((g_utf8_collate (supertype, "text") == 0) ||
			 (g_utf8_collate (supertype, "application") == 0)) {
			if (g_utf8_collate (item->type, "text/html") == 0) {
				vazaar_cmd_item_put_bookmark(app, item);
				pass = TRUE;
			}
			else {
				vazaar_cmd_item_put_text(app, item);	
				pass = TRUE;
			}
		}	
		
		/* Images */
		if (vazaar_cmd_strings_compare (supertype, "image") == 0) {
			vazaar_cmd_item_put_picture(app, item);
			pass = TRUE;
		}				

	g_free (supertype);
	}
	else {
		/* MIME not supported */
		vazaar_cmd_item_put_unknow_mime (app, item);
	}
}

void
vazaar_gui_item_get(Vazaar *app) {
    GtkTreeSelection *iselection = NULL;
	GtkTreeModel   *imodel;
	GtkTreeIter     iiter = {0};
	VazaarItem     *item;
	xmlNodePtr      node;
	gchar			*id;
	
	iselection = gtk_tree_view_get_selection(GTK_TREE_VIEW (app->treeitem));
	if (gtk_tree_selection_get_selected(iselection, &imodel, &iiter)) {
		if (&iiter) {
			/* select item by id and check if I must reload it */
			gtk_tree_model_get(imodel, &iiter, ITEM_COL_ID, &id, -1);		
			node = vazaar_xml_item_find(id);
			item = vazaar_xml_item_get(node);
			if ((g_utf8_collate (item->id, app->citem) != 0) || (app->force_reload)) {
				vazaar_gui_item_get_real (app, item);											
			}
			g_free (id);
		}
	}		
}

static gboolean 
gtk_source_buffer_load_with_encoding (GtkSourceBuffer *source_buffer,
				      const gchar     *filename,
				      const gchar     *encoding,
				      GError         **error)
{
	gchar		*title,
				*message;
	GIOChannel	*io;
	GtkTextIter	 iter;
	gchar		*buffer;
	gboolean	 reading;
	
	g_return_val_if_fail (source_buffer != NULL, FALSE);
	g_return_val_if_fail (filename != NULL, FALSE);
	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (source_buffer), FALSE);

	*error = NULL;

	io = g_io_channel_new_file (filename, "r", error);
	if (!io) {
		title = g_strdup_printf (_("Error %d: %s"), (*error)->code, (*error)->message);
		message = g_strdup_printf (_("libgtksourceview couldn't open:\n%s"), filename);
		vazaar_gui_show_dialog (NULL, GTK_STOCK_DIALOG_ERROR, title, message);
		return FALSE;
	}

	if (g_io_channel_set_encoding (io, encoding, error) != G_IO_STATUS_NORMAL) {
		title = g_strdup_printf (_("Error %d: %s"), (*error)->code, (*error)->message);
		message = g_strdup_printf (_("Failed to set encoding for:\n%s"), filename);
		vazaar_gui_show_dialog (NULL, GTK_STOCK_DIALOG_ERROR, title, message);
		return FALSE;
	}

	gtk_source_buffer_begin_not_undoable_action (source_buffer);

	gtk_text_buffer_set_text (GTK_TEXT_BUFFER (source_buffer), "", 0);
	buffer = g_malloc (READ_BUFFER_SIZE);
	reading = TRUE;
	while (reading)
	{
		gsize bytes_read;
		GIOStatus status;
		
		status = g_io_channel_read_chars (io, buffer,
						  READ_BUFFER_SIZE, &bytes_read,
						  error);
		switch (status)
		{
			case G_IO_STATUS_EOF:
				reading = FALSE;
				/* fall through */
				
			case G_IO_STATUS_NORMAL:
				if (bytes_read == 0)
				{
					continue;
				}
				
				gtk_text_buffer_get_end_iter (
					GTK_TEXT_BUFFER (source_buffer), &iter);
				gtk_text_buffer_insert (GTK_TEXT_BUFFER (source_buffer),
							&iter, buffer, bytes_read);
				break;
				
			case G_IO_STATUS_AGAIN:
				continue;

			case G_IO_STATUS_ERROR:
			default:
				title = g_strdup_printf (_("Error %d: %s"), (*error)->code, (*error)->message);
				message = g_strdup_printf (_("Because of error in input we clear already loaded text. File:\n%s"), filename);
				vazaar_gui_show_dialog (NULL, GTK_STOCK_DIALOG_ERROR, title, message);
				/* because of error in input we clear already loaded text */
				gtk_text_buffer_set_text (GTK_TEXT_BUFFER (source_buffer), "", 0);
				reading = FALSE;
				break;
		}
	}
	g_free (buffer);
	
	gtk_source_buffer_end_not_undoable_action (source_buffer);

	g_io_channel_unref (io);

	if (*error)
		return FALSE;

	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (source_buffer), FALSE);

	/* move cursor to the beginning */
	gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (source_buffer), &iter);
	gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (source_buffer), &iter);

	return TRUE;
}

static void
remove_all_markers (GtkSourceBuffer *buffer)
{
	GSList *markers;
	GtkTextIter begin, end;

	gtk_text_buffer_get_bounds (GTK_TEXT_BUFFER (buffer), &begin, &end);
	markers = gtk_source_buffer_get_markers_in_region (buffer, &begin, &end);
	while (markers)
	{
		GtkSourceMarker *marker = markers->data;
		
		gtk_source_buffer_delete_marker (buffer, marker);
		markers = g_slist_delete_link (markers, markers);
	}
}


/* Buffer creation -------------------------------------------------------------- */

static GtkSourceBuffer *
create_source_buffer (GtkSourceLanguagesManager *manager)
{
	GtkSourceBuffer *buffer;
	
	buffer = GTK_SOURCE_BUFFER (gtk_source_buffer_new (NULL));
	g_object_ref (manager);
	g_object_set_data_full (G_OBJECT (buffer), "languages-manager",
				manager, (GDestroyNotify) g_object_unref);
	
	return buffer;
}

void
vazaar_cmd_item_put_bookmark(Vazaar * app, VazaarItem * item)
{
    GtkMozEmbed		*gecko;
	GtkWidget		*statusbar,
					*box;
	GString			*data;
	
	//data = g_string_new ("<html><body>Hola mundo</body></html>");
	
    gecko = (GtkMozEmbed *) gtk_moz_embed_new();
	//gtk_moz_embed_render_data (gecko, data->str, data->len, "/", "text/html");
    gtk_moz_embed_load_url(gecko, item->data);
	
	//statusbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
	
	box = gtk_vbox_new (FALSE, 3);
	gtk_box_pack_start (GTK_BOX (box), GTK_WIDGET (gecko), TRUE, TRUE, 2);
	//gtk_box_pack_end (GTK_BOX (box), statusbar, FALSE, TRUE, 2);
	
    gtk_widget_show(GTK_WIDGET(gecko));
	//gtk_widget_show(GTK_WIDGET(statusbar));
	gtk_widget_show(GTK_WIDGET(box));
	
    gtk_paned_add2(GTK_PANED(app->vpitem), GTK_WIDGET(box));
    g_object_ref(box);
	
	if (DEBUG)
		g_message("%s => item->data: %s", __FUNCTION__, item->data);
}

void
vazaar_cmd_item_put_text(Vazaar * app, VazaarItem * item)
{
	GtkSourceLanguagesManager *manager;
	GtkSourceLanguage *language = NULL;
	GtkSourceBuffer *buffer = NULL;
	GError *err = NULL;
    GtkWidget      *text,
 					*sw;

	manager = gtk_source_languages_manager_new ();
	buffer = create_source_buffer (manager);

	if (item->type)
	{
		language = gtk_source_languages_manager_get_language_from_mime_type (manager,
										     item->type);
		if (language == NULL)
		{
			if (DEBUG)
				g_message ("No language found for mime type `%s'\n", item->type);
			g_object_set (G_OBJECT (buffer), "highlight", FALSE, NULL);
		}
		else
		{	
			g_object_set (G_OBJECT (buffer), "highlight", TRUE, NULL);

			gtk_source_buffer_set_language (buffer, language);
		}
	}
	else
	{
		g_object_set (G_OBJECT (buffer), "highlight", FALSE, NULL);

		g_warning ("Couldn't get mime type for file `%s'", item->data);
	}

	if (buffer) {
		remove_all_markers (buffer);
		if (!gtk_source_buffer_load_with_encoding (buffer, vazaar_cmd_item_file_id_get (item->id), "utf-8", &err)) {
			
			//vazaar_gui_show_dialog (app->window, GTK_STOCK_DIALOG_ERROR, ...) 
			//_("There was a problem in order to show this item"), _("Maybe, this item is not in UTF-8. To solve this problem convert this text into UTF-8 with your preferred editor"));
		}
		else {
			text = gtk_source_view_new_with_buffer (buffer);
			gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text), GTK_WRAP_WORD);
			gtk_source_view_set_show_line_numbers (GTK_SOURCE_VIEW(text), TRUE);
			gtk_source_view_set_show_line_markers (GTK_SOURCE_VIEW(text), TRUE);
			gtk_source_view_set_show_margin (GTK_SOURCE_VIEW(text), TRUE);
			gtk_text_view_set_editable (GTK_TEXT_VIEW (text), FALSE);
			set_monospace_font (text);
			sw = gtk_scrolled_window_new(NULL, NULL);
			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
						   GTK_POLICY_AUTOMATIC,
						   GTK_POLICY_AUTOMATIC);
		
			gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw), GTK_WIDGET(text));   
			gtk_paned_add2(GTK_PANED(app->vpitem), sw);
			gtk_widget_show(GTK_WIDGET(text));
			gtk_widget_show(sw);
			g_object_ref(sw);
		}
	}
}

void
vazaar_cmd_item_put_picture(Vazaar * app, VazaarItem * item)
{
    GdkPixbuf      *pixbuf;
    GtkWidget      *image;
    GError         *error = NULL;
    gint            wdg_width,	// widget width
                    wdg_height,	// widget height
                    scr_width,	// screen width
                    scr_height,	// screen height
                    hpn_width,	// hpaned position
                    vpn_height;	// vpaned position 

    /*
     * Obtenemos las medidas que va a tener el nuevo widget 
     */
    gtk_window_get_size(GTK_WINDOW(app->window), &scr_width, &scr_height);
    hpn_width = gtk_paned_get_position(GTK_PANED(app->hpcat));
    vpn_height = gtk_paned_get_position(GTK_PANED(app->vpitem));
    wdg_width = scr_width - hpn_width;
    wdg_height = scr_height - vpn_height;
    if (DEBUG)
		g_message("%s =>\nscreen(%d,%d)\npaned(%d,%d)\nwidget(%d,%d)",
			  __FUNCTION__, scr_width, scr_height, hpn_width,
			  vpn_height, wdg_width, wdg_height);

    pixbuf =
	gdk_pixbuf_new_from_file(vazaar_cmd_item_file_id_get(item->id),
				 &error);
    pixbuf = vazaar_cmd_pixbuf_scale_get(pixbuf, wdg_width, wdg_height);
    image = gtk_image_new_from_pixbuf(pixbuf);

    gtk_paned_add2(GTK_PANED(app->vpitem), image);
    gtk_widget_show(image);
    g_object_ref(image);
}

void
vazaar_cmd_item_put_unknow_mime 		(	Vazaar *app, 
											VazaarItem *item) {
	GtkWidget	*label;
	const gchar		*text;

	text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>", _("MIME not supported\nUse 'Info Panel' to launch its default application"));
	label = gtk_label_new (text);
	g_object_set (G_OBJECT (label), 
					"selectable", TRUE,
					"use-markup", TRUE, 
					"wrap", FALSE,
					"justify", GTK_JUSTIFY_CENTER,
		      NULL); 						
	//gtk_label_set_text (GTK_LABEL (label), text);									
	
	gtk_paned_add2(GTK_PANED(app->vpitem), label);
    gtk_widget_show(label);
    g_object_ref(label);
}
/*
	When you delete a category, we put its items in the wastebasket
*/
void
vazaar_cmd_move_cat_to_wastebasket (Vazaar *app, gchar *category) {
	//gchar			*i18n_wastebasket;
    VazaarItem		*item;
	xmlNodePtr		root, child;
	GSList			*update_items = NULL;
	
    item = (VazaarItem *) NULL;

    root = xmlDocGetRootElement(items_doc);
	child = root->xmlChildrenNode;
    while (child != NULL) {
		item = vazaar_xml_item_get(child);
		if (vazaar_cmd_item_belongto(category, item->categories)) {
			if (g_slist_length (item->categories) > 1) {
				// Delete from the current category /
				GSList *aux = item->categories;
				while (aux) {
					/* The only way that I could remove a category from a GSList */
					if (vazaar_cmd_strings_compare ((gchar *)aux->data, category) == 0){
						item->categories = g_slist_remove (item->categories, aux->data);
					}
					aux = aux->next;
				}
			}
			else {
				/* Move to wastebasket */
				item->categories = NULL;
				item->categories = g_slist_append (item->categories, "wastebasket");
			}
			update_items= g_slist_append (update_items, item);
		}
		child = child->next;
	}
	while (update_items) {
		VazaarItem *item = (VazaarItem *)update_items->data;
		vazaar_xml_item_delete (item->id);
		vazaar_xml_item_add (item);
		update_items = update_items->next;
	}
	vazaar_xml_items_savefile ();
}


void
vazaar_cmd_move_item_to_wastebasket (Vazaar *app, gchar *id) {
	xmlNodePtr	xItem;
	VazaarItem	*item;
	
	xItem = vazaar_xml_item_find (id);
	item = vazaar_xml_item_get (xItem);
	item->categories = NULL;
	item->categories = g_slist_append (item->categories, "wastebasket");
	vazaar_xml_item_delete (item->id);
	vazaar_xml_item_add (item);
}


void
vazaar_cmd_item_launch_defapp (GtkWidget *nothing, Vazaar *app) {
	GError	*error = NULL;		
	GnomeVFSURI *uri;
	gchar *guri;
	xmlNodePtr node = NULL;
	VazaarItem *item;
	gboolean test;
	gchar	*title,
			*message,
			*code;
	GList		*apps;
	
	node = vazaar_xml_item_find (app->citem);
	if (node) {
		item = vazaar_xml_item_get (node);
		if (g_utf8_collate (item->type, "text/html") == 0) {
			test = gnome_url_show (gnome_vfs_make_uri_canonical (item->data), &error);
		}
		else {
			apps = gnome_vfs_mime_get_all_applications (item->type);
			/*
			gnome_vfs_mime_get_default_action ()
			gnome_vfs_mime_get_default_action_type ()
			gnome_vfs_mime_get_default_application ()
			*/
			guri = vazaar_cmd_item_file_id_get (app->citem);
			uri = gnome_vfs_uri_new (guri);
			guri = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
			test = gnome_url_show (guri, &error);
		}
		if (test == FALSE) {
			code = g_strdup_printf (_("Error %d"), error->code);
			vazaar_gui_show_dialog (app->window, GTK_STOCK_DIALOG_ERROR, code, error->message);
		}
	}
}

/* From nautilus-text-view.c 
 *   Copyright (C) 2000 Eazel, Inc.
 *   Copyright (C) 2002 Sun Microsystems Inc.  */
void
set_monospace_font (GtkWidget *widget)
{
	GConfClient *conf_client;
	gchar        *monospace_font;

	/* Pick up the monospace font from desktop preferences */
	conf_client = gconf_client_get_default ();
	monospace_font = gconf_client_get_string (conf_client, "/desktop/gnome/interface/monospace_font_name", NULL);

	if (monospace_font != NULL) {
		PangoFontDescription *monospace_font_desc;
		monospace_font_desc = pango_font_description_from_string (monospace_font);
		gtk_widget_modify_font (widget, monospace_font_desc);
		pango_font_description_free (monospace_font_desc);
		g_free (monospace_font);
	}

	g_object_unref (conf_client);
}

void 
write_cb								(	GnomeVFSAsyncHandle *handle,
											GnomeVFSResult result,
											gconstpointer buffer,
											GnomeVFSFileSize bytes_requested,
											GnomeVFSFileSize bytes_written,
											gpointer callback_data) {
	g_message ("Hola write");											
}

void        
read_cb									(	GnomeVFSAsyncHandle *handle,
											GnomeVFSResult result,
											gpointer buffer,
											GnomeVFSFileSize bytes_requested,
											GnomeVFSFileSize bytes_read,
											gpointer callback_data) {

	g_message ("Hola read");	
}
