/*
 *  Vazaar
 *  file: items.h
 *
 *
 *  Copyright (C) 2005 Tomás Vírseda <kaskaras@yahoo.es>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _ITEMS_H
#define _ITEMS_H

#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlreader.h>
#include <libgnomevfs/gnome-vfs.h>
#include <libgnomevfs/gnome-vfs-mime-utils.h>
#include <gtkmozembed.h>
#include <gtksourceview/gtksourceview.h>
#include <gtksourceview/gtksourcebuffer.h>
#include <gtksourceview/gtksourcelanguage.h>
#include <gtksourceview/gtksourcelanguagesmanager.h>

typedef struct _VFSItemData VFSItemData;
typedef struct _VazaarDruidItem VazaarDruidItem;
typedef struct _VazaarItem VazaarItem;
typedef struct _VazaarItemSort VazaarItemSort;
enum {
	ITEM_COL_IMG,
    ITEM_COL_ID,
    ITEM_COL_NOM,
    ITEM_COL_MIME,
	ITEM_COL_DES,
	ITEM_COL_DATES,
	ITEM_COL_DATEM,
    ITEM_NUM_COLS
};

struct _VFSItemData {
    GnomeVFSURI    *uri_input,
                   *uri_output;
    GnomeVFSHandle *handle_input,
                   *handle_output;
    GnomeVFSResult  res;
    gchar          *file;
};

//GnomeVFSHandle

struct _VazaarItem {
    gchar		*id;		/* md5 sum */
    gchar		*type;		/* MIME Type (change this name) */
    gchar		*name;
    gchar		*desc;
    gchar		*data;		/* gnome-fvs uri:// or text */
	gchar		*dates;		/* Start date */
    gchar		*datem;		/* Modified date */
	gchar		*extra;		
    GSList		*categories;
};

struct _VazaarItemSort {
	VazaarItem	*item;
	GtkTreeIter	 iter;
};

struct _VazaarDruidItem {
    GladeXML		*xml;	/* Glade File */
    VazaarItem		*new_item;	/* Edit/New item */
    Vazaar			*app;	/* This Vazaar App */
    GtkWidget		*wparent,	/* Pointer to Main Window */
					*dialog,
					*itemName,	/* Item name widget */
					*itemDesc,	/* Item description widget */
					*rbLocal, 
					*rbRemote, 
					*rbCopypaste, 
					*frmLocal, 
					*frmRemote, 
					*frmCopypaste,
					*uri_remote,
					*uri_generic,
					*entry_uri_generic,
					*copypaste,
					*check_remote,
					*btnOK,
					*btnCancel,
					*treeviewcat;
    GSList			*cat_selected;
	gchar *uri;
    gboolean        edit;	/* edit_cat != NULL -> edit category */
};

/*
 * XML Functions 
 */

xmlNodePtr
vazaar_xml_item_set_node 				(	VazaarItem * item);

VazaarItem *
vazaar_xml_item_get						(	xmlNodePtr);

void
vazaar_xml_item_add						(	VazaarItem *);

gboolean
vazaar_xml_item_delete					(	const gchar *);

void            
vazaar_xml_item_edit					(	xmlNodePtr,
											xmlNodePtr);

xmlNodePtr      
vazaar_xml_item_find					(	gchar *);

int
vazaar_xml_items_savefile				(	void);

void vazaar_xml_item_change_category	(	gchar *, 
											gchar *);

/*
 * GUI Functions 
 */

void            
vazaar_gui_druiditem_delete_event		(	GtkWindow * win,
						  					VazaarDruidItem *);

GtkTreeModel *
vazaar_gui_itemmodel_create				();

gint
vazaar_gui_itemmodel_fill				(	Vazaar *app,
											GtkTreeModel *, 
											gchar *,
											GList **);

void
vazaar_gui_itemmodel_createview			(	GtkTreeModel *,
											GtkTreeView *);

void
vazaar_gui_itemmodel_refresh			(	GtkTreeView *);

gint
vazaar_gui_items_load					(	Vazaar *, 
											gchar *);

void            
vazaar_gui_item_add						(	Vazaar *);

void
vazaar_gui_item_edit					(	Vazaar *);

void            
vazaar_gui_item_del						(	Vazaar *);

void
vazaar_gui_druiditem_new				(	Vazaar *, 
											VazaarItem *,
											gchar *data_dragged);

//vazaar_gui_item_new_dragged (Vazaar *app, const gchar *uri);

void
vazaar_gui_druiditem_show				(	VazaarDruidItem *, 
											GtkWidget *);

void
vazaar_gui_druiditem_destroy			(	VazaarDruidItem *);

void            
vazaar_gui_druiditem_connect_signals	(	VazaarDruidItem *);

void            
vazaar_gui_druiditem_cancel				(	GtkWidget *,
					    					VazaarDruidItem *);

gboolean
vazaar_gui_druiditem_name_changed		(	GtkWidget *,
						  					VazaarDruidItem *);
gboolean            
vazaar_gui_druiditem_treecat_cursor_changed	(	GtkWidget *, 
												VazaarDruidItem *);
gboolean            
vazaar_gui_druiditem_uri_changed		(	GtkWidget *, 
											VazaarDruidItem *);

void
vazaar_gui_druiditem_finish				(	GtkWidget *,
											VazaarDruidItem *);

gboolean
vazaar_gui_druiditem_get_data			(	VazaarDruidItem *);

void 
vazaar_gui_item_select_mode				(	GtkWidget *,
											VazaarDruidItem *);

void vazaar_gui_item_change_href		(	GtkWidget *,
											VazaarDruidItem *);

void
vazaar_gui_item_set_info 				(	Vazaar *, 
											VazaarItem *);

gint 
vazaar_cmd_cat_sort_items 				(	VazaarItemSort *, 
											VazaarItemSort *);

/*
 * CMD Functions 
 */

VFSItemData *
vazaar_cmd_druiditem_item_store_create	(	VazaarDruidItem *);

void			
vazaar_cmd_druiditem_item_store_save	(	VFSItemData *);

gchar *          
vazaar_cmd_item_content_get				(	VazaarItem *);

gchar *         
vazaar_cmd_item_file_id_get				(	const gchar *);

void  
vazaar_cmd_item_delete					(	const gchar *);

void 
vazaar_gui_item_get_real 				(	Vazaar *, 
											VazaarItem *);
void            
vazaar_gui_item_get						(	Vazaar *);

void
vazaar_cmd_item_put_bookmark			(	Vazaar *,
					     					VazaarItem *);
											
void            
vazaar_cmd_item_put_text				(	Vazaar *, 
											VazaarItem *);
											
void            
vazaar_cmd_item_put_picture				(	Vazaar *,
					    					VazaarItem *);

void
vazaar_cmd_item_put_unknow_mime 		(	Vazaar *app, 
											VazaarItem *item);

gchar *
vazaar_cmd_itemtype_get					(	VazaarDruidItem *);

gboolean
vazaar_cmd_item_belongto				(	gchar *, 
											GSList *);
void
vazaar_cmd_druiditem_check 				(	GtkWidget *, 
											VazaarDruidItem *);

void
vazaar_cmd_move_cat_to_wastebasket		(Vazaar *, gchar *);

void
vazaar_cmd_move_item_to_wastebasket		(Vazaar *app, gchar *id);

void
vazaar_cmd_item_launch_defapp			(	GtkWidget *, Vazaar *);

gboolean
vazaar_cmd_item_filter_bad_mimes 		(	gchar *mime);

void
set_monospace_font						(	GtkWidget *);

void 
write_cb								(	GnomeVFSAsyncHandle *handle,
											GnomeVFSResult result,
											gconstpointer buffer,
											GnomeVFSFileSize bytes_requested,
											GnomeVFSFileSize bytes_written,
											gpointer callback_data);
										
void        
read_cb									(	GnomeVFSAsyncHandle *handle,
											GnomeVFSResult result,
											gpointer buffer,
											GnomeVFSFileSize bytes_requested,
											GnomeVFSFileSize bytes_read,
											gpointer callback_data);
#endif
